//--------------------------------------------------------------------------------------
// File: HDRSetup.cpp
// Author: Alexander Kharlamov
// Email: akharlamov@nvidia.com
//
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#pragma once
#pragma warning(disable: 4995)

#include "DXUT.h"
#include "SDKmisc.h"
#include "HDRSetup.h"
#include "FullScreenQuad.h"

ID3D10Texture2D*			g_pHDRTexture                 = NULL;
ID3D10ShaderResourceView*	g_pHDRTextureSRV              = NULL;;
ID3D10RenderTargetView*		g_pHDRTextureRTV              = NULL;

ID3D10Texture2D*			g_apToneMapTexture     [NUM_TONEMAP_TEXTURES]; 
ID3D10ShaderResourceView*	g_apToneMapTextureSRV  [NUM_TONEMAP_TEXTURES];
ID3D10RenderTargetView*		g_apToneMapTextureRTV  [NUM_TONEMAP_TEXTURES];

ID3D10Texture2D*			g_apBloomTexture       [NUM_BLOOM_TEXTURES];
ID3D10ShaderResourceView*	g_apBloomTextureSRV    [NUM_BLOOM_TEXTURES];
ID3D10RenderTargetView*		g_apBloomTextureRTV    [NUM_BLOOM_TEXTURES];

ID3D10Texture2D*            g_pBrightPassTexture          = NULL;
ID3D10ShaderResourceView*   g_pBrightPassTextureSRV       = NULL;
ID3D10RenderTargetView*     g_pBrightPassTextureRTV       = NULL;

ID3D10EffectShaderResourceVariable* g_pS0                 = NULL;
ID3D10EffectShaderResourceVariable* g_pS1                 = NULL;
ID3D10EffectShaderResourceVariable* g_pS2                 = NULL;

ID3D10EffectVectorVariable* g_pavOffsetsHorizontal        = NULL;
ID3D10EffectVectorVariable* g_pavOffsetsVertical          = NULL;
ID3D10EffectVectorVariable* g_pavSampleWeights            = NULL;

ID3D10Effect*               g_pHDREffect                  = NULL;
ID3D10EffectTechnique*      g_pDownScale                  = NULL;
ID3D10EffectTechnique*      g_pTMTechnique                = NULL; // Tone Mapping Technique
ID3D10EffectTechnique*      g_pBPTechnique                = NULL; // Bright Pass Technique
ID3D10EffectTechnique*      g_pBloomTechnique             = NULL; // Bloom Technique

ID3D10Texture2D*            g_HDRDepthTexture             = NULL;
ID3D10ShaderResourceView*   g_HDRDepthSRV                 = NULL;
ID3D10DepthStencilView*     g_HDRDepthDSV                 = NULL;

ID3D10RenderTargetView *g_pDefaultRTV                     = NULL;
ID3D10DepthStencilView *g_pDefaultDSV                     = NULL;
D3D10_VIEWPORT          g_DefaultVP;

bool g_RenderToHDR = false;

HRESULT InitHDREffect(ID3D10Device *pd3dDevice)
{
    HRESULT hr;
    WCHAR FilePath[MAX_PATH];
    WCHAR FileName[MAX_PATH] = L"..\\Source\\ParallaxMapping\\HDRSetup.fx";

    V_RETURN( DXUTFindDXSDKMediaFileCch( FilePath, MAX_PATH, FileName ) );
    V_RETURN( D3DX10CreateEffectFromFile( FilePath, NULL, NULL, "fx_4_0", D3D10_SHADER_NO_PRESHADER, 0, pd3dDevice, NULL, NULL, &g_pHDREffect, NULL, &hr ) );

	g_pBloomTechnique = g_pHDREffect->GetTechniqueByName( "Bloom"       );
	g_pTMTechnique    = g_pHDREffect->GetTechniqueByName( "ToneMapping" );
	g_pBPTechnique    = g_pHDREffect->GetTechniqueByName( "BrightPass"  );
	g_pDownScale      = g_pHDREffect->GetTechniqueByName( "DownScale"   );

	g_pS0 = g_pHDREffect->GetVariableByName("s0")->AsShaderResource();
    g_pS1 = g_pHDREffect->GetVariableByName("s1")->AsShaderResource();
    g_pS2 = g_pHDREffect->GetVariableByName("s2")->AsShaderResource();
	
	g_pavOffsetsHorizontal = g_pHDREffect->GetVariableByName( "g_avOffsetsHorizontal" )->AsVector();
	g_pavOffsetsVertical   = g_pHDREffect->GetVariableByName( "g_avOffsetsVertical"   )->AsVector();
    g_pavSampleWeights     = g_pHDREffect->GetVariableByName( "g_avSampleWeights"     )->AsVector();
    
    D3D10_PASS_DESC passDesc;

	g_pTMTechnique->GetPassByIndex(0)->GetDesc(&passDesc);

    V_RETURN( InitFullScreenQuad(pd3dDevice, passDesc) );

    return hr;
}

HRESULT InitHDR(ID3D10Device *pd3dDevice, unsigned int Width, unsigned int Height)
{
    HRESULT hr;

    V_RETURN( InitHDREffect( pd3dDevice ) );
//  Create HDR: Texture2D, Shader Resource View, Render Target View 
//  Create HDR: Texture2D
    D3D10_TEXTURE2D_DESC t2dHDR;
    ZeroMemory( &t2dHDR, sizeof( D3D10_TEXTURE2D_DESC ) );
	t2dHDR.Width          = Width;
    t2dHDR.Height         = Height;
    t2dHDR.Format         = DXGI_FORMAT_R16G16B16A16_FLOAT;
    t2dHDR.MipLevels      = 1;
    t2dHDR.ArraySize      = 1;
    t2dHDR.Usage          = D3D10_USAGE_DEFAULT;
    t2dHDR.BindFlags      = D3D10_BIND_SHADER_RESOURCE | D3D10_BIND_RENDER_TARGET;
    t2dHDR.SampleDesc.Count   = 1;
	V_RETURN( pd3dDevice->CreateTexture2D( &t2dHDR, NULL, &g_pHDRTexture ) );
//  Create HDR: Shader Resource View
	D3D10_SHADER_RESOURCE_VIEW_DESC  srvHDR;
	srvHDR.Format = DXGI_FORMAT_R16G16B16A16_FLOAT;
    srvHDR.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
    srvHDR.Texture2D.MipLevels = 1;
    srvHDR.Texture2D.MostDetailedMip = 0;
    V_RETURN( pd3dDevice->CreateShaderResourceView( g_pHDRTexture, &srvHDR, &g_pHDRTextureSRV ) );
//  Create HDR:  Render Target View 
	D3D10_RENDER_TARGET_VIEW_DESC rtvHDR;
    rtvHDR.Format = DXGI_FORMAT_R16G16B16A16_FLOAT;
    rtvHDR.ViewDimension = D3D10_RTV_DIMENSION_TEXTURE2D;
    rtvHDR.Texture2D.MipSlice = 0;
	V_RETURN( pd3dDevice->CreateRenderTargetView( g_pHDRTexture, &rtvHDR, &g_pHDRTextureRTV ) );

//  Create Tone Map (TM): Texture2D, Shader Resource View, Render Target View 
	int nSampleLen = 1;
	for( int i=0; i < NUM_TONEMAP_TEXTURES; i++ )
	{
//      Create Tone Map (TM): Texture2D
        D3D10_TEXTURE2D_DESC t2dTM;
        ZeroMemory( &t2dTM, sizeof( D3D10_TEXTURE2D_DESC ) );
        t2dTM.ArraySize = 1;
        t2dTM.BindFlags = D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE;
        t2dTM.Usage     = D3D10_USAGE_DEFAULT;
        t2dTM.Format    = DXGI_FORMAT_R16_FLOAT;
        t2dTM.Width     = nSampleLen;
        t2dTM.Height    = nSampleLen;
        t2dTM.MipLevels = 1;
        t2dTM.SampleDesc.Count = 1;
        V_RETURN( pd3dDevice->CreateTexture2D( &t2dTM, NULL, &g_apToneMapTexture[i] ) );
//      Create Tone Map (TM): Render Target View 
        D3D10_RENDER_TARGET_VIEW_DESC rtvTM;
        rtvTM.Format = t2dTM.Format;
        rtvTM.ViewDimension = D3D10_RTV_DIMENSION_TEXTURE2D;
        rtvTM.Texture2D.MipSlice = 0;
        V_RETURN(pd3dDevice->CreateRenderTargetView( g_apToneMapTexture[i], &rtvTM, &g_apToneMapTextureRTV[i] ));
//      Create Tone Map (TM): Shader Resource View
        D3D10_SHADER_RESOURCE_VIEW_DESC srvTM;
        srvTM.Format = t2dTM.Format;
        srvTM.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
        srvTM.Texture2D.MipLevels = 1;
        srvTM.Texture2D.MostDetailedMip = 0;
        V_RETURN(pd3dDevice->CreateShaderResourceView( g_apToneMapTexture[i], &srvTM, &g_apToneMapTextureSRV[i] ));

        nSampleLen *= 3;
    }

//  Create Bright Pass: Texture2D, Render Target View, Shader Resource View
//  Create Bright Pass: Texture2D
    D3D10_TEXTURE2D_DESC t2dBP;
    ZeroMemory( &t2dBP, sizeof( D3D10_TEXTURE2D_DESC ) );
    t2dBP.ArraySize = 1;
    t2dBP.BindFlags = D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE;
    t2dBP.Usage     = D3D10_USAGE_DEFAULT;
    t2dBP.Format    = DXGI_FORMAT_R8G8B8A8_UNORM;
    t2dBP.Width     = Width / 8;
    t2dBP.Height    = Height / 8;
    t2dBP.MipLevels = 1;
    t2dBP.SampleDesc.Count = 1;
    V_RETURN( pd3dDevice->CreateTexture2D( &t2dBP, NULL, &g_pBrightPassTexture ) );
//  Create Bright Pass: Render Target View
    D3D10_RENDER_TARGET_VIEW_DESC rtvBP;
	rtvBP.Format = t2dBP.Format;
    rtvBP.ViewDimension = D3D10_RTV_DIMENSION_TEXTURE2D;
    rtvBP.Texture2D.MipSlice = 0;
    V_RETURN(pd3dDevice->CreateRenderTargetView( g_pBrightPassTexture, &rtvBP, &g_pBrightPassTextureRTV ));
//  Create Bright Pass: Shader Resource View
    D3D10_SHADER_RESOURCE_VIEW_DESC srvBP;
	srvBP.Format = t2dBP.Format;
    srvBP.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
    srvBP.Texture2D.MipLevels = 1;
    srvBP.Texture2D.MostDetailedMip = 0;
    V_RETURN(pd3dDevice->CreateShaderResourceView( g_pBrightPassTexture, &srvBP, &g_pBrightPassTextureSRV ));

//  Create Bloom: Texture2D, Rendeer Target View, Shader Resource View
    for( int i=0; i < NUM_BLOOM_TEXTURES; i++ )
    {
//      Create Bloom: Texture2D
        D3D10_TEXTURE2D_DESC t2dBloom;
        ZeroMemory( &t2dBloom, sizeof( D3D10_TEXTURE2D_DESC ) );
        t2dBloom.ArraySize = 1;
        t2dBloom.BindFlags = D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE;
        t2dBloom.Usage     = D3D10_USAGE_DEFAULT;
        t2dBloom.Format    = DXGI_FORMAT_R8G8B8A8_UNORM;
        t2dBloom.Width     = Width / 8;
        t2dBloom.Height    = Height / 8;
        t2dBloom.MipLevels = 1;
        t2dBloom.SampleDesc.Count = 1;
        V_RETURN( pd3dDevice->CreateTexture2D( &t2dBloom, NULL, &g_apBloomTexture[i] ) );
//      Create Bloom: Render Target View
        D3D10_RENDER_TARGET_VIEW_DESC rtvBP;
        rtvBP.Format = t2dBloom.Format;
        rtvBP.ViewDimension = D3D10_RTV_DIMENSION_TEXTURE2D;
        rtvBP.Texture2D.MipSlice = 0;
        V_RETURN(pd3dDevice->CreateRenderTargetView( g_apBloomTexture[i], &rtvBP, &g_apBloomTextureRTV[i] ));
//      Create Bloom: Shader Resource View
        D3D10_SHADER_RESOURCE_VIEW_DESC srvBP;
        srvBP.Format = t2dBloom.Format;
        srvBP.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
        srvBP.Texture2D.MipLevels = 1;
        srvBP.Texture2D.MostDetailedMip = 0;
        V_RETURN(pd3dDevice->CreateShaderResourceView( g_apBloomTexture[i], &srvBP, &g_apBloomTextureSRV[i] ));
    }

    V_RETURN( CreateDepthBufferTexture(pd3dDevice, Width, Height) );

	return S_OK;
}

HRESULT MeasureLuminance(ID3D10Device *pd3dDevice)
{
    ID3D10EffectShaderResourceVariable* pS0;
    ID3D10EffectShaderResourceVariable* pS1;
    pS0 = g_pS0;
    pS1 = g_pS1;

    ID3D10ShaderResourceView* pSource   = NULL;
    ID3D10RenderTargetView*   pDestination = NULL;

    //-------------------------------------------------------------------------
    // Initial sampling pass to convert the image to the log of the grayscale
    //-------------------------------------------------------------------------
    pSource      = g_pHDRTextureSRV;
    pDestination = g_apToneMapTextureRTV[NUM_TONEMAP_TEXTURES-1];

    D3D10_TEXTURE2D_DESC descSrc; 
    g_pHDRTexture->GetDesc( &descSrc );

    D3D10_TEXTURE2D_DESC descDest;
    g_apToneMapTexture[NUM_TONEMAP_TEXTURES-1]->GetDesc( &descDest );

    ID3D10RenderTargetView* aRTViews[ 1 ] = { pDestination };
    pd3dDevice->OMSetRenderTargets( 1, aRTViews, NULL );
    pS0->SetResource( pSource );

	g_pDownScale->GetPassByName( "DownScale2x2" )->Apply( 0 ); 
	
    DrawFullScreenQuad( pd3dDevice, descDest.Width, descDest.Height );

    pS0->SetResource( NULL );

    //-------------------------------------------------------------------------
    // Iterate through the remaining tone map textures
    //------------------------------------------------------------------------- 
    for( int i = NUM_TONEMAP_TEXTURES - 1; i > 0; i-- )
    {
        // Cycle the textures
        pSource = g_apToneMapTextureSRV[i];
        pDestination = g_apToneMapTextureRTV[i-1];

        D3D10_TEXTURE2D_DESC desc;
        g_apToneMapTexture[i]->GetDesc( &desc );

        aRTViews[ 0 ] = pDestination;
        pd3dDevice->OMSetRenderTargets( 1, aRTViews, NULL );

        pS0->SetResource( pSource );
  
		g_pDownScale->GetPassByName( "DownScale3x3" )->Apply( 0 );

        DrawFullScreenQuad( pd3dDevice, desc.Width/3, desc.Height/3 );
       
        pS0->SetResource( NULL );
    }

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: BrightPassFilter
// Desc: Prepare for the bloom pass by removing dark information from the scene
//-----------------------------------------------------------------------------
HRESULT BrightPassFilter(ID3D10Device *pd3dDevice)
{
    ID3D10EffectShaderResourceVariable* pS0;
    ID3D10EffectShaderResourceVariable* pS1;
    pS0 = g_pS0;
    pS1 = g_pS1;

    const DXGI_SURFACE_DESC* pBackBufDesc = DXUTGetDXGIBackBufferSurfaceDesc();

    ID3D10RenderTargetView* aRTViews[ 1 ] = { g_pBrightPassTextureRTV };
    pd3dDevice->OMSetRenderTargets( 1, aRTViews, NULL );

    pS0->SetResource( g_pHDRTextureSRV );
    pS1->SetResource( g_apToneMapTextureSRV[0] );

	g_pBPTechnique->GetPassByIndex(0)->Apply(0);
	
    DrawFullScreenQuad( pd3dDevice, pBackBufDesc->Width / 8, pBackBufDesc->Height / 8 );

    pS0->SetResource( NULL );
    pS1->SetResource( NULL );

    return S_OK;
}

//--------------------------------------------------------------------------------------
float GaussianDistribution( float x, float y, float rho )
{
    float g = 1.0f / sqrtf( 2.0f * D3DX_PI * rho * rho );
    g *= expf( -(x*x + y*y)/(2*rho*rho) );

    return g;
}

//-----------------------------------------------------------------------------
// Name: GetSampleOffsets_Bloom()
// Desc: 
//-----------------------------------------------------------------------------
HRESULT GetSampleOffsetsForBloom(float TexSize, float afTexCoordOffset[15], D3DXVECTOR4* avColorWeight, float fDeviation, float fMultiplier )
{
    int i = 0;
    float tu = 1.0f / (float)TexSize;

    // Fill the center texel
    float weight = 1.0f * GaussianDistribution( 0, 0, fDeviation );
    avColorWeight[7] = D3DXVECTOR4( weight, weight, weight, 1.0f );

    afTexCoordOffset[7] = 0.0f;
    
    // Fill the right side
    for( i=1; i < 8; i++ )
    {
        weight = fMultiplier * GaussianDistribution( (float)i, 0, fDeviation );
        afTexCoordOffset[7-i] = i * tu;

        avColorWeight[7-i] = D3DXVECTOR4( weight, weight, weight, 1.0f );
    }

    // Copy to the left side
    for( i=8; i < 15; i++ )
    {
        avColorWeight[i]    =  avColorWeight[14-i];
        afTexCoordOffset[i] = -afTexCoordOffset[14-i];
    }

    return S_OK;
}

//-----------------------------------------------------------------------------
// Desc: Render the blooming effect
//-----------------------------------------------------------------------------
HRESULT RenderBloom(ID3D10Device* pd3dDevice)
{
    HRESULT hr = S_OK;
    int i=0;

    ID3D10EffectShaderResourceVariable* pS0;
    ID3D10EffectShaderResourceVariable* pS1;
    pS0 = g_pS0;
    pS1 = g_pS1;

    D3DXVECTOR4 avSampleOffsets[15];
    float       afSampleOffsets[15];
    D3DXVECTOR4 avSampleWeights[15];

    // Vertical Blur
    ID3D10RenderTargetView* pDestination = g_apBloomTextureRTV[1];
    D3D10_TEXTURE2D_DESC RTDesc;
    g_apBloomTexture[1]->GetDesc( &RTDesc );
    
    D3D10_TEXTURE2D_DESC desc;
    g_pBrightPassTexture->GetDesc( &desc );

    V_RETURN( GetSampleOffsetsForBloom( (float)desc.Width, afSampleOffsets, avSampleWeights, 3.0f, 1.25f ) );
    
	for( i = 0; i < 15; i++ )
    {
        avSampleOffsets[i] = D3DXVECTOR4( afSampleOffsets[i], 0.0f, 0.0f, 0.0f );
    }
     
    g_pavOffsetsHorizontal->SetFloatVectorArray( (float*)avSampleOffsets, 0, 15 );
    g_pavSampleWeights->SetFloatVectorArray( (float*)avSampleWeights, 0, 15 );
   
    ID3D10RenderTargetView* aRTViews[ 1 ] = { pDestination };
    pd3dDevice->OMSetRenderTargets( 1, aRTViews, NULL );

    pS0->SetResource( g_pBrightPassTextureSRV );
    g_pBloomTechnique->GetPassByIndex( 0 )->Apply( 0 );
	DrawFullScreenQuad( pd3dDevice, RTDesc.Width, RTDesc.Height );
    
	pS0->SetResource( NULL );
    ID3D10ShaderResourceView* pViews[4] = {0,0,0,0};
    pd3dDevice->PSSetShaderResources( 0, 4, pViews );

    // Horizontal Blur
    pDestination = g_apBloomTextureRTV[0];
    
    hr = GetSampleOffsetsForBloom( (float)desc.Height, afSampleOffsets, avSampleWeights, 3.0f, 1.25f );
    for( i = 0; i < 15; i++ )
    {
        avSampleOffsets[i] = D3DXVECTOR4( 0.0f, afSampleOffsets[i], 0.0f, 0.0f );
    }

    g_pavOffsetsVertical->SetRawValue( avSampleOffsets, 0, sizeof(avSampleOffsets) );
    g_pavSampleWeights->SetRawValue( avSampleWeights, 0, sizeof(avSampleWeights) );
   
    aRTViews[ 0 ] = pDestination;
    pd3dDevice->OMSetRenderTargets( 1, aRTViews, NULL );

    pS0->SetResource( g_apBloomTextureSRV[1] );       
    g_pBloomTechnique->GetPassByIndex( 1 )->Apply( 0 );
	DrawFullScreenQuad( pd3dDevice, RTDesc.Width, RTDesc.Height );
    pS0->SetResource( NULL );

    return hr;
}

void ReleaseHDR(void)
{
    SAFE_RELEASE( g_pHDREffect      );

    SAFE_RELEASE( g_pHDRTexture     );
    SAFE_RELEASE( g_pHDRTextureRTV  );
    SAFE_RELEASE( g_pHDRTextureSRV  );

    SAFE_RELEASE( g_pBrightPassTexture    );
    SAFE_RELEASE( g_pBrightPassTextureRTV );
    SAFE_RELEASE( g_pBrightPassTextureSRV );

    for (int i = 0; i < NUM_BLOOM_TEXTURES; i++)
    {
        SAFE_RELEASE( g_apBloomTexture[i]    );
        SAFE_RELEASE( g_apBloomTextureSRV[i] );
        SAFE_RELEASE( g_apBloomTextureRTV[i] );
    }

    for (int i = 0; i < NUM_TONEMAP_TEXTURES; i++)
    {
        SAFE_RELEASE( g_apToneMapTexture[i]    );
        SAFE_RELEASE( g_apToneMapTextureRTV[i] );
        SAFE_RELEASE( g_apToneMapTextureSRV[i] );
    }

    DeleteDepthBufferTexture();
    
    ReleaseFullScreenQuad();
}


bool g_HDREnabled = false;

void    RenderToHDR(ID3D10Device *pd3dDevice)
{
    D3D10_TEXTURE2D_DESC desc;
	g_pDefaultRTV = DXUTGetD3D10RenderTargetView();	
    g_pDefaultDSV = DXUTGetD3D10DepthStencilView();

	// Set all critical states for main passes
	g_pHDREffect->GetTechniqueByName( "SetHDRStates" )->GetPassByIndex( 0 )->Apply( 0 );
    
    UINT NumViewports = 1;
    pd3dDevice->RSGetViewports( &NumViewports, &g_DefaultVP );

    pd3dDevice->OMSetRenderTargets( 1, &g_pHDRTextureRTV, g_HDRDepthDSV );
	g_pHDRTexture->GetDesc( &desc );

    float clearColor[] = {1.0f, 1.0f, 1.0f, 1.0f };

    pd3dDevice->ClearRenderTargetView( g_pHDRTextureRTV, clearColor );
    pd3dDevice->ClearDepthStencilView( g_HDRDepthDSV, D3D10_CLEAR_DEPTH, 1.0, 0 );
    pd3dDevice->RSSetViewports( 1, &g_DefaultVP );

    g_HDREnabled = true;
}

void    RestoreFromHDR(ID3D10Device *pd3dDevice)
{
    g_HDREnabled = false;
}

void ShowHDR(ID3D10Device *pd3dDevice)
{
	pd3dDevice->OMSetRenderTargets( 1, &g_pHDRTextureRTV, NULL );
	pd3dDevice->RSSetViewports( 1, &g_DefaultVP );
		
	MeasureLuminance(pd3dDevice);
	BrightPassFilter(pd3dDevice);
	
	if (g_RenderToHDR)
		RenderBloom     (pd3dDevice);

	ID3D10RenderTargetView* RTViews[ 1 ] = { g_pDefaultRTV };
	pd3dDevice->OMSetRenderTargets( 1, RTViews, g_pDefaultDSV );

	g_pS0->SetResource( g_pHDRTextureSRV );
	g_pS1->SetResource( g_apToneMapTextureSRV[0] );
	g_pS2->SetResource( g_RenderToHDR ? g_apBloomTextureSRV[0] : NULL );
	
	g_pTMTechnique->GetPassByIndex( 0 )->Apply( 0 );
	DrawFullScreenQuad( pd3dDevice );
}

void DeleteDepthBufferTexture(void)
{
    SAFE_RELEASE( g_HDRDepthTexture );
	SAFE_RELEASE( g_HDRDepthSRV );
	SAFE_RELEASE( g_HDRDepthDSV );
}

HRESULT CreateDepthBufferTexture( ID3D10Device* pd3dDevice, UINT width, UINT height )
{
	HRESULT hr = S_OK;
	DeleteDepthBufferTexture();

	D3D10_TEXTURE2D_DESC desc;
    desc.Width = width;
    desc.Height = height;
	desc.Format = DXGI_FORMAT_R32_TYPELESS; // Format of the main depth buffer
    desc.MipLevels = 1;
    desc.ArraySize = 1;
    desc.Usage = D3D10_USAGE_DEFAULT;
    desc.BindFlags = D3D10_BIND_SHADER_RESOURCE | D3D10_BIND_DEPTH_STENCIL;
    desc.MiscFlags = 0;
    desc.CPUAccessFlags = 0;
    desc.SampleDesc.Count = 1;
    desc.SampleDesc.Quality = 0;
	V_RETURN( pd3dDevice->CreateTexture2D( &desc, NULL, &g_HDRDepthTexture ) );
	
    D3D10_SHADER_RESOURCE_VIEW_DESC SRVdesc;
	SRVdesc.Format = DXGI_FORMAT_R32_FLOAT;
    SRVdesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
    SRVdesc.Texture2D.MipLevels = 1;
    SRVdesc.Texture2D.MostDetailedMip = 0;
	V_RETURN( pd3dDevice->CreateShaderResourceView( g_HDRDepthTexture, &SRVdesc, &g_HDRDepthSRV ) );

    D3D10_DEPTH_STENCIL_VIEW_DESC DSVdesc;
    DSVdesc.Format = DXGI_FORMAT_D32_FLOAT;
    DSVdesc.ViewDimension = D3D10_DSV_DIMENSION_TEXTURE2D;
    DSVdesc.Texture2D.MipSlice = 0;
	V_RETURN( pd3dDevice->CreateDepthStencilView( g_HDRDepthTexture, &DSVdesc, &g_HDRDepthDSV ) );

	return hr;
}