//--------------------------------------------------------------------------------------
// File: ParallaxMapping.cpp
// Author: Alexander Kharlamov
// Email: akharlamov@nvidia.com
//
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------
#pragma warning(disable: 4995)

#include "DXUT.h"
#include "DXUTgui.h"
#include "DXUTmisc.h"
#include "DXUTCamera.h"
#include "NVUTmesh.h"
#include "DXUTSettingsDlg.h"
#include "sdkmesh.h"
#include "sdkmisc.h"
#include "defines.h"

#include "HDRSetup.h"

// full set of textures for relief mapping: diffuse, bump and height
ID3D10Texture2D*                                           g_pT2DDiffuse[MAX_MESH_NUMBER];
ID3D10Texture2D*                                           g_pT2DHeight[MAX_MESH_NUMBER];
ID3D10Texture2D*                                           g_pT2DBump[MAX_MESH_NUMBER];
ID3D10Texture2D*                                           g_pT2DCubeMapDepth   = NULL;

ID3D10ShaderResourceView*                                  g_pSRVDiffuse[MAX_MESH_NUMBER];
ID3D10ShaderResourceView*                                  g_pSRVHeight[MAX_MESH_NUMBER];
ID3D10ShaderResourceView*                                  g_pSRVBump[MAX_MESH_NUMBER];
ID3D10ShaderResourceView*                                  g_pSRVCubeMapDepth   = NULL;

ID3D10EffectShaderResourceVariable*                        g_pESRVDiffuse       = NULL;
ID3D10EffectShaderResourceVariable*                        g_pESRVHeight        = NULL;
ID3D10EffectShaderResourceVariable*                        g_pESRVBump          = NULL;
ID3D10EffectShaderResourceVariable*                        g_pESRVCubeMapDepth  = NULL;

ID3D10DepthStencilView*                                    g_pDSVCubeMapDepth[6]= { NULL, NULL, NULL, NULL, NULL, NULL };

ID3D10EffectMatrixVariable*                                g_pEMVModel          = NULL;
ID3D10EffectMatrixVariable*                                g_pEMVViewProj       = NULL;
ID3D10EffectMatrixVariable*                                g_pEMVWorldViewProj  = NULL;
ID3D10EffectMatrixVariable*                                g_pEMVCubeViewMatrix = NULL;
ID3D10EffectMatrixVariable*                                g_pEMVCubeProjMatrix = NULL;

ID3D10EffectVectorVariable*                                g_pEVVCameraEye      = NULL;
ID3D10EffectVectorVariable*                                g_pEVVLightPos       = NULL;
ID3D10EffectVectorVariable*                                g_pEVVTexScale       = NULL;

ID3D10EffectScalarVariable*                                g_pESVExtrudeFin     = NULL;
ID3D10EffectScalarVariable*                                g_pESVExtrudeAngle   = NULL;
ID3D10EffectScalarVariable*                                g_pESVCoarseStepsMax = NULL;
ID3D10EffectScalarVariable*                                g_pESVDetailStepsMax = NULL;
ID3D10EffectScalarVariable*                                g_pESVCoarseStepsMin = NULL;
ID3D10EffectScalarVariable*                                g_pESVDetailStepsMin = NULL;
ID3D10EffectScalarVariable*                                g_pESVParallaxClamp  = NULL;
ID3D10EffectScalarVariable*                                g_pESVSign           = NULL;
ID3D10EffectScalarVariable*                                g_pESVTime           = NULL;

// Debug values
ID3D10EffectScalarVariable*                                g_pESVBoolean1       = NULL;
ID3D10EffectScalarVariable*                                g_pESVBoolean2       = NULL;
ID3D10EffectScalarVariable*                                g_pESVBoolean3       = NULL;

ID3D10Resource*                                            g_pResource          = NULL;

// Effect and Techniques
ID3D10Effect*                                              g_pMeshEffect        = NULL;
ID3D10EffectTechnique*                                     g_pMeshTechnique     = NULL;
ID3D10EffectTechnique*                                     g_pReliefMapping     = NULL;
ID3D10EffectTechnique*                                     g_pBumpMapping       = NULL;
ID3D10EffectTechnique*                                     g_pSilhouette        = NULL;
ID3D10EffectTechnique*                                     g_pBoxTechnique      = NULL;

// Direct3D 10 HUD resources
ID3DX10Font*                                               g_pFont   = NULL;
ID3DX10Sprite*                                             g_pSprite = NULL;
CDXUTDialogResourceManager                                 g_DialogResourceManager; // manager for shared resources of dialogs
CD3DSettingsDlg                                            g_SettingsDlg;           // device settings dialog
CDXUTDialog                                                g_HUD;                   // dialog for standard controls
CDXUTDialog                                                g_GUI;    

// InputLayouts for Mesh and Box
ID3D10InputLayout*                                         g_pMeshLayout  = NULL;
ID3D10InputLayout*                                         g_pBoxLayout   = NULL;

#define SAFE_DESTROY_MESH(p)  { if (p) { (p)->Destroy(); SAFE_DELETE(p); } }
CDXUTSDKMesh*                                              g_pMesh = NULL;
CDXUTSDKMesh*                                              g_pBox  = NULL;

CFirstPersonCamera                                         g_Camera;

D3DXMATRIX  g_pmCubeViewMatrices [6];
D3DXMATRIX  g_mCubeProjectionMatrix;
D3DXVECTOR4 g_LightPos(0, 0, 0, 1);

const float g_LightRadius = 35;
const float g_LightHeight = 25;

D3DXVECTOR3 g_EyePos  (0, 0, 0);
D3DXVECTOR3 g_EyeDir  (0, 0, 0);

int g_CameraSpeed = 1;

WCHAR g_wcExtrudeFin[MAX_WCHAR_SIZE]       = L"Extrude fin: ";			// 13 char-s
float g_fExtrudeFin = 0.10f;
float g_fExtrudeAngle = 0.01f;
WCHAR g_wcCoarseStepsMax[MAX_WCHAR_SIZE]   = L"Coarse steps Max: ";     
float g_fCoarseStepsMax = 10.0f;
WCHAR g_wcDetailStepsMax[MAX_WCHAR_SIZE]   = L"Detail steps Max: ";     
float g_fDetailStepsMax = 6.0f;
WCHAR g_wcCoarseStepsMin[MAX_WCHAR_SIZE]   = L"Coarse steps Min: ";   
float g_fCoarseStepsMin = 6.0f;
WCHAR g_wcDetailStepsMin[MAX_WCHAR_SIZE]   = L"Detail steps Min: ";  
float g_fDetailStepsMin = 2.0f;
WCHAR g_wcParallaxClamp[MAX_WCHAR_SIZE]    = L"Parallax Clamp: ";
float g_fParallaxClamp = 2.0f;

float g_fSign = 1.0f;

bool  g_PauseLight = true;

float g_Boolean1 = 1.0f;
float g_Boolean2 = 1.0f;
float g_Boolean3 = 1.0f;

struct SimpleVertex
{
	D3DXVECTOR3 Position;
	D3DXVECTOR2 TexCoord;
};

D3DXMATRIX g_mView;
D3DXMATRIX g_mModel;
D3DXMATRIX g_mProj;

D3DXMATRIX g_mScale;
D3DXMATRIX g_Translate;

void InitCubeMatrices( float * cubeCenter, int side );
int  InitCubeMap     (ID3D10Device *pd3dDevice);

void RenderText(void);
void RenderShadowMap (ID3D10Device *pd3dDevice, int i);
void RenderMesh      (ID3D10Device *pd3dDevice);
void RenderBox       (ID3D10Device *pd3dDevice);
void RotateLight(float * lightPos, bool isPaused);

void RotateLight( float * lightPos, bool isPaused, float fElapsedTime )
{
    D3DXMATRIX spinX, spinY;

	static float fTime = 0.0f;
	static const float fSpeed = 0.05f;

	if (!isPaused)
	{
		fTime += fSpeed * fElapsedTime;
	}

	lightPos[0] = g_LightRadius * sin(fTime);
	lightPos[2] = g_LightRadius * cos(fTime);
    lightPos[1] = 0; //g_LightHeight * cos(fTime);

	g_pEVVLightPos->SetFloatVector(lightPos);
}

void RenderShadowMap(ID3D10Device *pd3dDevice, int i)
{
	// SAVE CURRENT RENDER TARGET and VIEWPORT
	ID3D10RenderTargetView *pCurrentRT = NULL;
	ID3D10DepthStencilView *pCurrentDS = NULL;
	D3D10_VIEWPORT currentVP;
	unsigned int NumVP = 1;

	D3D10_VIEWPORT cubeVP;
	cubeVP.Height = CUBE_MAP_SIZE;
	cubeVP.Width  = CUBE_MAP_SIZE;
	cubeVP.MinDepth = 0;
	cubeVP.MaxDepth = 1;
	cubeVP.TopLeftX = 0;
	cubeVP.TopLeftY = 0;

	pd3dDevice->OMGetRenderTargets(1, &pCurrentRT, &pCurrentDS);
	pd3dDevice->RSGetViewports(&NumVP, &currentVP);

	// BIND RENDER TARGET FOR SHADOW MAPS
	ID3D10RenderTargetView *pNullRT = NULL;
	pd3dDevice->OMSetRenderTargets(1, &pNullRT, g_pDSVCubeMapDepth[i]);

	pd3dDevice->RSSetViewports(1, &cubeVP);
	pd3dDevice->ClearDepthStencilView( g_pDSVCubeMapDepth[i], D3D10_CLEAR_DEPTH, 1.0, 0 );

	InitCubeMatrices( (float *) &g_LightPos, i );

	D3DXMATRIX mViewProj = g_pmCubeViewMatrices[i] * g_mCubeProjectionMatrix;
    g_pEMVViewProj->SetMatrix ( (float* ) &mViewProj );

	RenderMesh( pd3dDevice );

	pd3dDevice->OMSetRenderTargets(1, &pCurrentRT, pCurrentDS);
	pd3dDevice->RSSetViewports(NumVP, &currentVP);

	SAFE_RELEASE( pCurrentDS );
	SAFE_RELEASE( pCurrentRT );
}

void RenderMesh(ID3D10Device *pd3dDevice)
{
	static D3DXVECTOR3 s(0.1f, 0.1f, 0.1f);
	static D3DXVECTOR3 t(0.0f, 0.0f, 0.0f);

    D3DXMATRIX mS;
    D3DXMATRIX mT;

	D3DXMatrixScaling    ( &mS, s.x, s.y, s.z );
    D3DXMatrixTranslation( &mT, t.x, t.y, t.z );

    D3DXMATRIX mM = mS * mT;
	g_pEMVModel->SetMatrix( (float * ) &mM );

    pd3dDevice->IASetInputLayout(g_pMeshLayout);
    pd3dDevice->IASetIndexBuffer(NULL,DXGI_FORMAT_R32_UINT,0);

    g_pESRVDiffuse->SetResource ( g_pSRVDiffuse[MESH] );
    g_pESRVBump->SetResource    ( g_pSRVBump[MESH]    );
    g_pESRVHeight->SetResource  ( g_pSRVHeight[MESH]  );

    g_pMesh->Render(pd3dDevice, g_pMeshTechnique);
}
void RenderBox(ID3D10Device *pd3dDevice)
{
	static D3DXVECTOR3 s(100.0f, 100.0f, 100.0f);
	static D3DXVECTOR3 t(  0.0f,   0.0f,   0.0f);
	D3DXMATRIX mS;
    D3DXMATRIX mT;
    D3DXMatrixScaling    ( &mS, s.x, s.y, s.z );
    D3DXMatrixTranslation( &mT, t.x, t.x, t.z );

	D3DXMATRIX mM = mS * mT;
	g_pEMVModel->SetMatrix( (float* ) &mM );

	unsigned int strides[1];
    unsigned int offsets[1];

    strides [0] = sizeof( SimpleVertex );
    offsets [0] = 0;
    
	D3D10_TECHNIQUE_DESC techDesc;
	g_pBoxTechnique->GetDesc( &techDesc );

	pd3dDevice->IASetInputLayout(g_pBoxLayout);
	g_pBox->Render( pd3dDevice, g_pBoxTechnique );
}

int InitCubeMap( ID3D10Device* pd3dDevice )
{
    // Create cubic depth stencil texture.
    D3D10_TEXTURE2D_DESC dstex;

    dstex.Width  = CUBE_MAP_SIZE;
    dstex.Height = CUBE_MAP_SIZE;
    dstex.MipLevels = 1;
    dstex.ArraySize = 6;
    dstex.SampleDesc.Count = 1;
    dstex.SampleDesc.Quality = 0;
	dstex.Format = DXGI_FORMAT_R32_TYPELESS;
    dstex.Usage  = D3D10_USAGE_DEFAULT;
	dstex.BindFlags = D3D10_BIND_DEPTH_STENCIL | D3D10_BIND_SHADER_RESOURCE;
    dstex.CPUAccessFlags = 0;
    dstex.MiscFlags = D3D10_RESOURCE_MISC_TEXTURECUBE;

	if( FAILED( pd3dDevice->CreateTexture2D( &dstex, NULL, &g_pT2DCubeMapDepth ) ) )
    {
        DXUTTRACE( L"Failed to create depth stencil texture\n" );
        return false;
    }

    // Create the depth stencil view for the entire cube
    D3D10_DEPTH_STENCIL_VIEW_DESC DescDS;
	for (int DepthStencilSlice = 0; DepthStencilSlice < 6; DepthStencilSlice++)
	{   
		DescDS.Format = DXGI_FORMAT_D32_FLOAT;
	    DescDS.ViewDimension = D3D10_DSV_DIMENSION_TEXTURE2DARRAY;
	    DescDS.Texture2DArray.MipSlice = 0;
		DescDS.Texture2DArray.FirstArraySlice = DepthStencilSlice;
		DescDS.Texture2DArray.ArraySize = 1;

		if( FAILED( pd3dDevice->CreateDepthStencilView( g_pT2DCubeMapDepth, &DescDS, &g_pDSVCubeMapDepth[DepthStencilSlice] ) ) )
		{
			DXUTTRACE( L"Failed to create depth stencil view for a depth cube map\n" );
			return false;
		}
	}

    // Create the shader resource view for the shadow map
    D3D10_SHADER_RESOURCE_VIEW_DESC SRVDesc;

    ZeroMemory( &SRVDesc, sizeof( SRVDesc ) );

	SRVDesc.Format = DXGI_FORMAT_R32_FLOAT;
	SRVDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURECUBE;
	SRVDesc.TextureCube.MipLevels = 1;
    SRVDesc.TextureCube.MostDetailedMip = 0;

    if( FAILED( pd3dDevice->CreateShaderResourceView( g_pT2DCubeMapDepth, &SRVDesc, &g_pSRVCubeMapDepth ) ) )
    {
        DXUTTRACE( L"Failed to create shader resource view for a depth stencil\n" );
        return false;
    }

    return true;
}

void InitCubeMatrices( float * cubeCenter, int shadow_side )
{
    D3DXVECTOR3 vLookDir;
    D3DXVECTOR3 vUpDir;
	switch (shadow_side)
	{
		case 0:
			vLookDir = D3DXVECTOR3( 1.0f, 0.0f, 0.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
			vUpDir   = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
			D3DXMatrixLookAtLH( &g_pmCubeViewMatrices[0], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );
		break;
		case 1:
			vLookDir = D3DXVECTOR3( -1.0f, 0.0f, 0.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
			vUpDir   = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
			D3DXMatrixLookAtLH( &g_pmCubeViewMatrices[1], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );
		break;
		case 2:
			vLookDir = D3DXVECTOR3( 0.0f, 1.0f,  0.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
			vUpDir   = D3DXVECTOR3( 0.0f, 0.0f, -1.0f );
			D3DXMatrixLookAtLH( &g_pmCubeViewMatrices[2], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );
		break;
		case 3:
			vLookDir = D3DXVECTOR3( 0.0f, -1.0f, 0.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
			vUpDir   = D3DXVECTOR3( 0.0f,  0.0f, 1.0f );
			D3DXMatrixLookAtLH( &g_pmCubeViewMatrices[3], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );
		break;
		case 4:
			vLookDir = D3DXVECTOR3( 0.0f, 0.0f, 1.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
			vUpDir   = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
			D3DXMatrixLookAtLH( &g_pmCubeViewMatrices[4], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );
		break;
		case 5:
			vLookDir = D3DXVECTOR3( 0.0f, 0.0f, -1.0f ) + (* (D3DXVECTOR3 *) cubeCenter);
			vUpDir   = D3DXVECTOR3( 0.0f, 1.0f,  0.0f );
			D3DXMatrixLookAtLH( &g_pmCubeViewMatrices[5], (D3DXVECTOR3 *) cubeCenter, &vLookDir, &vUpDir );
		break;
	}

    D3DXMatrixPerspectiveFovLH( &g_mCubeProjectionMatrix, (float)D3DX_PI * 0.5f, 1.0f, 0.1f, 100.0f );
}

void SetTechniqueByIndex(int nTechnique)
{
	switch (nTechnique)
	{
		case 0: g_pMeshTechnique = g_pSilhouette;
			break;
		case 1: g_pMeshTechnique = g_pReliefMapping;
			break;
		case 2: g_pMeshTechnique = g_pBumpMapping;
			break;
	}
}
void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext )
{
    switch( nControlID )
    {
        case IDC_TOGGLEFULLSCREEN: DXUTToggleFullScreen();
			break;
        case IDC_TOGGLEREF:        DXUTToggleREF(); 
			break;
        case IDC_CHANGEDEVICE:     g_SettingsDlg.SetActive( !g_SettingsDlg.IsActive() ); 
			break;
		
		case IDC_EXTRUDE_FIN_SLIDER:
			g_fExtrudeFin = (float) (g_GUI.GetSlider( IDC_EXTRUDE_FIN_SLIDER )->GetValue() * 0.01f);	    // Get the value from GUI slider
			g_pESVExtrudeFin->SetFloat( g_fExtrudeFin );													// Update current value of g_ExtrudeFin
			swprintf(&g_wcExtrudeFin[13], MAX_WCHAR_SIZE, L" %5.2f", g_fExtrudeFin);						// Update g_wcExtrudeFin string
			g_GUI.GetStatic( IDC_EXTRUDE_FIN_STATIC )->SetText(g_wcExtrudeFin);								// Update GUI caption
			break;

		case IDC_COARSE_STEPS_MAX_SLIDER:
			g_fCoarseStepsMax = (float) (g_GUI.GetSlider( IDC_COARSE_STEPS_MAX_SLIDER )->GetValue());		// Get the value from GUI slider
			g_pESVCoarseStepsMax->SetFloat( g_fCoarseStepsMax );
			swprintf(&g_wcCoarseStepsMax[17], MAX_WCHAR_SIZE, L" %5.2f", g_fCoarseStepsMax);				// Update g_wcCoarseStepsMax string
			g_GUI.GetStatic( IDC_COARSE_STEPS_MAX_STATIC )->SetText(g_wcCoarseStepsMax);	                // Update GUI caption
			break;

		case IDC_DETAIL_STEPS_MAX_SLIDER:
			g_fDetailStepsMax = (float) (g_GUI.GetSlider( IDC_DETAIL_STEPS_MAX_SLIDER )->GetValue());		// Get the value from GUI slider
			g_pESVDetailStepsMax->SetFloat( g_fDetailStepsMax );
			swprintf(&g_wcDetailStepsMax[17], MAX_WCHAR_SIZE, L" %5.2f", g_fDetailStepsMax);				// Update g_wcDetailStepsMax string
			g_GUI.GetStatic( IDC_DETAIL_STEPS_MAX_STATIC )->SetText(g_wcDetailStepsMax);	                // Update GUI caption
			break;

		case IDC_COARSE_STEPS_MIN_SLIDER:
			g_fCoarseStepsMin = (float) (g_GUI.GetSlider( IDC_COARSE_STEPS_MIN_SLIDER )->GetValue());		// Get the value from GUI slider
			g_pESVCoarseStepsMin->SetFloat( g_fCoarseStepsMin );
			swprintf(&g_wcCoarseStepsMin[17], MAX_WCHAR_SIZE, L" %5.2f", g_fCoarseStepsMin);				// Update g_wcCoarseStepsMax string
			g_GUI.GetStatic( IDC_COARSE_STEPS_MIN_STATIC )->SetText(g_wcCoarseStepsMin);	                // Update GUI caption
			break;

		case IDC_DETAIL_STEPS_MIN_SLIDER:
			g_fDetailStepsMin = (float) (g_GUI.GetSlider( IDC_DETAIL_STEPS_MIN_SLIDER )->GetValue());		// Get the value from GUI slider
			g_pESVDetailStepsMin->SetFloat( g_fDetailStepsMin );
			swprintf(&g_wcDetailStepsMin[17], MAX_WCHAR_SIZE, L" %5.2f", g_fDetailStepsMin);				// Update g_wcDetailStepsMax string
			g_GUI.GetStatic( IDC_DETAIL_STEPS_MIN_STATIC )->SetText(g_wcDetailStepsMin);	                // Update GUI caption
			break;

		case IDC_PARALLAX_CLAMP_SLIDER:
			g_fParallaxClamp = (float) (g_GUI.GetSlider( IDC_PARALLAX_CLAMP_SLIDER )->GetValue() * 0.1f);	// Get the value from GUI slider
			g_pESVParallaxClamp->SetFloat( g_fParallaxClamp );
			swprintf(&g_wcParallaxClamp[16], MAX_WCHAR_SIZE, L" %5.2f", g_fParallaxClamp);					// Update g_wcParallaxClamp string
			g_GUI.GetStatic( IDC_PARALLAX_CLAMP_STATIC )->SetText(g_wcParallaxClamp);	                    // Update GUI caption
			break;

        case IDC_TECHNIQUES_COMBO:
            {
                CDXUTComboBox* pComboBox = g_GUI.GetComboBox(IDC_TECHNIQUES_COMBO);
                const int nTechnique = pComboBox->GetSelectedIndex();
                SetTechniqueByIndex(nTechnique);
            }
            break;

        case IDC_HDR_RENDERING:
			g_RenderToHDR = !g_RenderToHDR;
            break;
        case IDC_MOVE_LIGHT:
			g_PauseLight = !g_PauseLight;
            break;
    }
}

HRESULT InitDialog()
{
    int iY = 10;
	int iX = 35;
	int iW = 150;
	int iH = 22;

    g_SettingsDlg.Init ( &g_DialogResourceManager );
    g_GUI.Init         ( &g_DialogResourceManager );
    g_HUD.Init         ( &g_DialogResourceManager );

    g_HUD.SetCallback( OnGUIEvent ); 
    g_HUD.AddButton( IDC_TOGGLEFULLSCREEN, L"Toggle full screen", iX, iY, iW, iH );
    g_HUD.AddButton( IDC_TOGGLEREF,        L"Toggle REF (F3)",    iX, iY += 25, iW, iH, VK_F3 );
    g_HUD.AddButton( IDC_CHANGEDEVICE,     L"Change device (F2)", iX, iY += 25, iW, iH, VK_F2 );

    CDXUTComboBox* pComboBox = NULL;

    g_GUI.SetCallback( OnGUIEvent );   
    g_GUI.AddStatic  ( IDC_MODE_STATIC, L"(R)ender mode", iX, iY += 25, iW, iH );
    g_GUI.AddComboBox( IDC_TECHNIQUES_COMBO, iX, iY += 25, iW, iH, 'R', false, &pComboBox );

    if( pComboBox )
    {
        pComboBox->SetDropHeight( 60 );
        pComboBox->SetHotkey('R' );
    }

    const int nTechs = 3;
    const WCHAR* RENDER_TECHNIQUES[nTechs] =
    {
        L"Silhouette Clipping",
		L"Relief Mapping",
		L"Bump Mapping",
    };

    for( int i = 0; i < nTechs; ++i )
        pComboBox->AddItem( RENDER_TECHNIQUES[i], IntToPtr(i) );
    
	g_GUI.AddStatic( IDC_EXTRUDE_FIN_STATIC, g_wcExtrudeFin, iX, iY += 25, iW, iH); 
    g_GUI.AddSlider( IDC_EXTRUDE_FIN_SLIDER, iX, iY += 25, iW, iH, 1, 100, (int) (g_fExtrudeFin * 100) );
	swprintf(&g_wcExtrudeFin[13], MAX_WCHAR_SIZE, L" %5.2f", g_fExtrudeFin);						        // Update g_wcExtrudeFin string
	g_GUI.GetStatic( IDC_EXTRUDE_FIN_STATIC )->SetText(g_wcExtrudeFin);	                                    // Update GUI caption
	
	g_GUI.AddStatic( IDC_COARSE_STEPS_MAX_STATIC, g_wcCoarseStepsMax, iX, iY += 25, iW, iH); 
    g_GUI.AddSlider( IDC_COARSE_STEPS_MAX_SLIDER, iX, iY += 25, iW, iH, 1, 64, (int) (g_fCoarseStepsMax) );
	swprintf(&g_wcCoarseStepsMax[17], MAX_WCHAR_SIZE, L" %5.2f", g_fCoarseStepsMax);						// Update g_wcCoarseStepsMax string
	g_GUI.GetStatic( IDC_COARSE_STEPS_MAX_STATIC )->SetText(g_wcCoarseStepsMax);	                        // Update GUI caption
	
	g_GUI.AddStatic( IDC_DETAIL_STEPS_MAX_STATIC, g_wcDetailStepsMax, iX, iY += 25, iW, iH); 
	g_GUI.AddSlider( IDC_DETAIL_STEPS_MAX_SLIDER, iX, iY += 25, iW, iH, 1, 32, (int) (g_fDetailStepsMax) );
	swprintf(&g_wcDetailStepsMax[17], MAX_WCHAR_SIZE, L" %5.2f", g_fDetailStepsMax);						// Update g_wcDetailStepsMax string
	g_GUI.GetStatic( IDC_DETAIL_STEPS_MAX_STATIC )->SetText(g_wcDetailStepsMax);	                        // Update GUI caption

#ifdef N_DOT_V_LERPING
	g_GUI.AddStatic( IDC_COARSE_STEPS_MIN_STATIC, g_wcCoarseStepsMin, iX, iY += 25, iW, iH); 
    g_GUI.AddSlider( IDC_COARSE_STEPS_MIN_SLIDER, iX, iY += 25, iW, iH, 1, 64, (int) (g_fCoarseStepsMin) );
	swprintf(&g_wcCoarseStepsMin[17], MAX_WCHAR_SIZE, L" %5.2f", g_fCoarseStepsMin);						// Update g_wcCoarseStepsMax string
	g_GUI.GetStatic( IDC_COARSE_STEPS_MIN_STATIC )->SetText(g_wcCoarseStepsMin);	                        // Update GUI caption
	
	g_GUI.AddStatic( IDC_DETAIL_STEPS_MIN_STATIC, g_wcDetailStepsMin, iX, iY += 25, iW, iH); 
	g_GUI.AddSlider( IDC_DETAIL_STEPS_MIN_SLIDER, iX, iY += 25, iW, iH, 1, 32, (int) (g_fDetailStepsMin) );
	swprintf(&g_wcDetailStepsMin[17], MAX_WCHAR_SIZE, L" %5.2f", g_fDetailStepsMin);						// Update g_wcDetailStepsMax string
	g_GUI.GetStatic( IDC_DETAIL_STEPS_MIN_STATIC )->SetText(g_wcDetailStepsMin);
#endif

	g_GUI.AddStatic( IDC_PARALLAX_CLAMP_STATIC, g_wcParallaxClamp, iX, iY += 25, iW, iH); 
	g_GUI.AddSlider( IDC_PARALLAX_CLAMP_SLIDER, iX, iY += 25, iW, iH, 1, 30, (int) (g_fParallaxClamp * 10) );
	swprintf(&g_wcParallaxClamp[16], MAX_WCHAR_SIZE, L" %5.2f", g_fParallaxClamp);					        // Update g_wcParallaxClamp string
	g_GUI.GetStatic( IDC_PARALLAX_CLAMP_STATIC )->SetText(g_wcParallaxClamp);	                            // Update GUI caption

    CDXUTCheckBox* pCheckBox = NULL;

    g_GUI.AddCheckBox(IDC_HDR_RENDERING, L"Render (B)loom", iX, iY += 25, iW, iH, g_RenderToHDR, 'B', false, &pCheckBox);

    g_GUI.AddCheckBox(IDC_MOVE_LIGHT, L"(P)ause spot light", iX, iY += 25, iW, iH, g_PauseLight, 'P', false, &pCheckBox);

    return true;
}

HRESULT InitMesh( CDXUTSDKMesh *pMesh, ID3D10InputLayout **ppInputLayout, LPCWSTR mesh, ID3D10Device* pd3dDevice )
{
    HRESULT hr;

    if (pMesh != NULL) pMesh->Destroy();

    const D3D10_INPUT_ELEMENT_DESC meshLayout[] =
    {
        { "POSITION",  0, DXGI_FORMAT_R32G32B32_FLOAT, 0,  0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "NORMAL",    0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "TEXCOORD",  0, DXGI_FORMAT_R32G32_FLOAT,    0, 24, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "TANGENT",   0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 32, D3D10_INPUT_PER_VERTEX_DATA, 0 }
    };

    D3D10_PASS_DESC PassDesc;

    V_RETURN( g_pMeshTechnique->GetPassByIndex( 0 )->GetDesc( &PassDesc ) );
    V_RETURN( pd3dDevice->CreateInputLayout( meshLayout, sizeof(meshLayout)/sizeof(D3D10_INPUT_ELEMENT_DESC), PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, ppInputLayout ) );
    
	pMesh->Create(pd3dDevice, mesh, true);

    return S_OK;
}

HRESULT InitPOMEffect( LPCWSTR effect, ID3D10Device* pd3dDevice )
{
    HRESULT hr;
    WCHAR path[MAX_PATH] = L"";

    SAFE_RELEASE( g_pMeshEffect );

    V_RETURN(DXUTFindDXSDKMediaFileCch(path, MAX_PATH, effect));
    V_RETURN(D3DX10CreateEffectFromFile(path, NULL, NULL, "fx_4_0", D3D10_SHADER_PREFER_FLOW_CONTROL, 0, pd3dDevice, NULL, NULL, &g_pMeshEffect, NULL, &hr));

	g_pReliefMapping = g_pMeshEffect->GetTechniqueByName( "ReliefMapping" );
    g_pBumpMapping   = g_pMeshEffect->GetTechniqueByName( "BumpMapping" );
    g_pSilhouette    = g_pMeshEffect->GetTechniqueByName( "SilhouetteClipping" );
	g_pBoxTechnique  = g_pMeshEffect->GetTechniqueByName( "RenderBox" );
    
	g_pMeshTechnique = g_pSilhouette;

    g_pEMVWorldViewProj  = g_pMeshEffect->GetVariableBySemantic( "ModelViewProjection" )->AsMatrix();
    g_pEMVViewProj       = g_pMeshEffect->GetVariableBySemantic( "ViewProjection"      )->AsMatrix();
    g_pEMVModel          = g_pMeshEffect->GetVariableBySemantic( "Model"               )->AsMatrix();
	g_pEMVCubeViewMatrix = g_pMeshEffect->GetVariableBySemantic( "CubeView"            )->AsMatrix();
	g_pEMVCubeProjMatrix = g_pMeshEffect->GetVariableBySemantic( "CubeProj"            )->AsMatrix();
    
    g_pESRVDiffuse      = g_pMeshEffect->GetVariableByName( "t2dDiffuse"   )->AsShaderResource();
    g_pESRVBump         = g_pMeshEffect->GetVariableByName( "t2dBump"      )->AsShaderResource();
    g_pESRVHeight       = g_pMeshEffect->GetVariableByName( "t2dHeight"    )->AsShaderResource();
	g_pESRVCubeMapDepth = g_pMeshEffect->GetVariableByName( "t2dShadowMap" )->AsShaderResource();

    g_pEVVCameraEye = g_pMeshEffect->GetVariableByName( "g_CameraEye" )->AsVector();
	g_pEVVLightPos = g_pMeshEffect->GetVariableByName(  "g_LightPos"  )->AsVector();
	g_pEVVLightPos->SetFloatVector( (float*) g_LightPos );
	
	g_pEVVTexScale = g_pMeshEffect->GetVariableByName( "g_TexScale" )->AsVector();
	float texShift[] = { 2.0f, 20.0f };
	g_pEVVTexScale->SetFloatVector( (float*) texShift );

	g_pESVSign = g_pMeshEffect->GetVariableByName("g_Sign")->AsScalar();
	g_pESVSign->SetFloat(g_fSign);

	g_pESVBoolean1 = g_pMeshEffect->GetVariableByName("g_Boolean1")->AsScalar();
	g_pESVBoolean1->SetFloat(g_Boolean1);
	g_pESVBoolean2 = g_pMeshEffect->GetVariableByName("g_Boolean2")->AsScalar();
	g_pESVBoolean2->SetFloat(g_Boolean2);
	g_pESVBoolean3 = g_pMeshEffect->GetVariableByName("g_Boolean3")->AsScalar();
	g_pESVBoolean3->SetFloat(g_Boolean3);

	g_pESVExtrudeFin = g_pMeshEffect->GetVariableByName("g_ExtrudeFin")->AsScalar();
	g_pESVExtrudeFin->SetFloat(g_fExtrudeFin);

	g_pESVExtrudeAngle = g_pMeshEffect->GetVariableByName("g_ExtrudeAngle")->AsScalar();
	g_pESVExtrudeAngle->SetFloat(g_fExtrudeAngle);

	g_pESVCoarseStepsMax = g_pMeshEffect->GetVariableByName("g_CoarseStepsMax")->AsScalar();
	g_pESVCoarseStepsMax->SetFloat(g_fCoarseStepsMax);

	g_pESVDetailStepsMax = g_pMeshEffect->GetVariableByName("g_DetailStepsMax")->AsScalar();
	g_pESVDetailStepsMax->SetFloat(g_fDetailStepsMax);

	g_pESVCoarseStepsMin = g_pMeshEffect->GetVariableByName("g_CoarseStepsMin")->AsScalar();
	g_pESVCoarseStepsMin->SetFloat(g_fCoarseStepsMin);

	g_pESVDetailStepsMin = g_pMeshEffect->GetVariableByName("g_DetailStepsMin")->AsScalar();
	g_pESVDetailStepsMin->SetFloat(g_fDetailStepsMin);

	g_pESVParallaxClamp = g_pMeshEffect->GetVariableByName("g_ParallaxClamp")->AsScalar();
	g_pESVParallaxClamp->SetFloat(g_fParallaxClamp);

    g_pESRVDiffuse->SetResource     ( g_pSRVDiffuse[MESH] );
    g_pESRVBump->SetResource        ( g_pSRVBump[MESH]    );
    g_pESRVHeight->SetResource      ( g_pSRVHeight[MESH]  );

    return S_OK;
}

void InitCamera()
{
    // Setup the camera's view parameters
    D3DXVECTOR3 vecEye( 1.5f, -2.0f, 9.5f );
    D3DXVECTOR3 vecAt ( 1.5f, -2.0f, 8.5f );

    g_Camera.SetEnablePositionMovement( true );
    g_Camera.SetEnableYAxisMovement( true );
    g_Camera.SetScalers();
    g_Camera.SetViewParams( &vecEye, &vecAt );

}

HRESULT InitTexture( LPCWSTR wcDiffuse, LPCWSTR wcBump, LPCWSTR wcHeight, int texNum, ID3D10Device* pd3dDevice )
{
    HRESULT hr;
    D3D10_SHADER_RESOURCE_VIEW_DESC SRVdesc;
    D3D10_TEXTURE2D_DESC            T2Ddesc;
    WCHAR path[MAX_PATH];

    SAFE_RELEASE( g_pT2DDiffuse[texNum] );
    SAFE_RELEASE( g_pT2DBump[texNum]    );
    SAFE_RELEASE( g_pT2DHeight[texNum]  );

    V_RETURN( DXUTFindDXSDKMediaFileCch  ( path, MAX_PATH, wcDiffuse )                       );
    V_RETURN( D3DX10CreateTextureFromFile( pd3dDevice, path, NULL, NULL, &g_pResource, &hr ) );
    g_pResource->QueryInterface( __uuidof( ID3D10Texture2D ), (LPVOID*)&g_pT2DDiffuse[texNum] );
    g_pResource->Release();

    V_RETURN( DXUTFindDXSDKMediaFileCch  ( path, MAX_PATH, wcBump )                          );
    V_RETURN( D3DX10CreateTextureFromFile( pd3dDevice, path, NULL, NULL, &g_pResource, &hr ) );
    g_pResource->QueryInterface( __uuidof( ID3D10Texture2D ), (LPVOID*)&g_pT2DBump[texNum]    );
    g_pResource->Release();

    V_RETURN( DXUTFindDXSDKMediaFileCch  ( path, MAX_PATH, wcHeight )                        );
    V_RETURN( D3DX10CreateTextureFromFile( pd3dDevice, path, NULL, NULL, &g_pResource, &hr ) );
    g_pResource->QueryInterface( __uuidof( ID3D10Texture2D ), (LPVOID*)&g_pT2DHeight[texNum]  );
    g_pResource->Release();

    SAFE_RELEASE( g_pSRVDiffuse[texNum] );
    SAFE_RELEASE( g_pSRVBump[texNum]    );
    SAFE_RELEASE( g_pSRVHeight[texNum]  );

    // Diffuse Texture Shader Resource View
    g_pT2DDiffuse[texNum]->GetDesc( &T2Ddesc );
    ZeroMemory( &SRVdesc, sizeof(SRVdesc) );
    SRVdesc.Format              = T2Ddesc.Format;
    SRVdesc.ViewDimension       = D3D10_SRV_DIMENSION_TEXTURE2D;
    SRVdesc.Texture2D.MipLevels = T2Ddesc.MipLevels;

    V_RETURN(pd3dDevice->CreateShaderResourceView( g_pT2DDiffuse[texNum], &SRVdesc, &g_pSRVDiffuse[texNum] ));
    
    // Bump Texture Shader Resource View
    g_pT2DBump[texNum]->GetDesc( &T2Ddesc );
    ZeroMemory( &SRVdesc, sizeof(SRVdesc) );
    SRVdesc.Format              = T2Ddesc.Format;
    SRVdesc.ViewDimension       = D3D10_SRV_DIMENSION_TEXTURE2D;
    SRVdesc.Texture2D.MipLevels = T2Ddesc.MipLevels;

    V_RETURN(pd3dDevice->CreateShaderResourceView( g_pT2DBump[texNum], &SRVdesc, &g_pSRVBump[texNum] ));
    
    // Height Texture Shader Resource View
    g_pT2DHeight[texNum]->GetDesc( &T2Ddesc );
    ZeroMemory( &SRVdesc, sizeof(SRVdesc) );
    SRVdesc.Format              = T2Ddesc.Format;
    SRVdesc.ViewDimension       = D3D10_SRV_DIMENSION_TEXTURE2D;
    SRVdesc.Texture2D.MipLevels = T2Ddesc.MipLevels;

    V_RETURN(pd3dDevice->CreateShaderResourceView( g_pT2DHeight[texNum], &SRVdesc, &(g_pSRVHeight[texNum]) ));

    return S_OK;
}

void RenderText()
{
    CDXUTTextHelper txtHelper(g_pFont, g_pSprite, 15);
    txtHelper.Begin();
    txtHelper.SetInsertionPos( 5, 5 );
    txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 0.0f, 1.0f ) );
    txtHelper.DrawTextLine( DXUTGetFrameStats(true) );
    txtHelper.DrawTextLine( DXUTGetDeviceStats() );
    txtHelper.End();
}

//--------------------------------------------------------------------------------------
// Reject any D3D10 devices that aren't acceptable by returning false
//--------------------------------------------------------------------------------------
bool CALLBACK IsD3D10DeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType, DXGI_FORMAT BackBufferFormat, bool bWindowed, void* pUserContext )
{
    return true;
}

//--------------------------------------------------------------------------------------
// Called right before creating a D3D9 or D3D10 device, allowing the app to modify the device settings as needed
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext )
{
    // For the first device created if its a REF device, optionally display a warning dialog box
    static bool s_bFirstTime = true;
    if( s_bFirstTime )
    {
        s_bFirstTime = false;
        if( (DXUT_D3D9_DEVICE == pDeviceSettings->ver && pDeviceSettings->d3d9.DeviceType == D3DDEVTYPE_REF) ||
            (DXUT_D3D10_DEVICE == pDeviceSettings->ver && pDeviceSettings->d3d10.DriverType == D3D10_DRIVER_TYPE_REFERENCE) )
        DXUTDisplaySwitchingToREFWarning( pDeviceSettings->ver );
    }

    return true;
}

//--------------------------------------------------------------------------------------
// Create any D3D10 resources that aren't dependant on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10CreateDevice( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr;

    V_RETURN( g_DialogResourceManager.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( g_SettingsDlg.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( D3DX10CreateFont( pd3dDevice, 15, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, L"Arial", &g_pFont ) );      

    InitTexture( L"ParallaxMapping\\rock_diffuse.dds", 
                 L"ParallaxMapping\\rock_bump.dds", 
                 L"ParallaxMapping\\rock_height.dds", MESH, pd3dDevice );

    InitTexture( L"ParallaxMapping\\rockwall_diffuse.dds", 
                 L"ParallaxMapping\\rockwall_bump.dds", 
                 L"ParallaxMapping\\rockwall_height.dds", CUBE, pd3dDevice );
                 
    InitPOMEffect( L"ParallaxMapping.fx", pd3dDevice );

    g_pMesh = new CDXUTSDKMesh();
    g_pBox  = new CDXUTSDKMesh();
	
    InitMesh   ( g_pMesh, &g_pMeshLayout, L"ParallaxMapping\\tor.sdkmesh", pd3dDevice );    
    InitMesh   ( g_pBox,  &g_pBoxLayout,  L"ParallaxMapping\\box.sdkmesh", pd3dDevice );    

	InitCubeMap(pd3dDevice);

    InitCamera();

    return S_OK;
}


//--------------------------------------------------------------------------------------
// Create any D3D10 resources that depend on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10ResizedSwapChain( ID3D10Device* pd3dDevice, IDXGISwapChain *pSwapChain, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr;

    V_RETURN( g_DialogResourceManager.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) );
    V_RETURN( g_SettingsDlg.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) );

    float fAspectRatio = pBackBufferSurfaceDesc->Width / (FLOAT)pBackBufferSurfaceDesc->Height;
    g_Camera.SetProjParams( D3DX_PI / 3, fAspectRatio, 0.1f, 1000.0f );

    g_HUD.SetLocation( pBackBufferSurfaceDesc->Width - 220, 0);
	g_GUI.SetLocation( pBackBufferSurfaceDesc->Width - 220, 0);

    V_RETURN( InitHDR( pd3dDevice, pBackBufferSurfaceDesc->Width, pBackBufferSurfaceDesc->Height ) );

    return S_OK;
}


//--------------------------------------------------------------------------------------
// Handle updates to the scene.  This is called regardless of which D3D API is used
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext )
{
    g_Camera.FrameMove( fElapsedTime * g_CameraSpeed );
}


//--------------------------------------------------------------------------------------
// Render the scene using the D3D10 device
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10FrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
	static const float texScaleMesh[] = { 2.0f, 20.0f };
	static const float texScaleShadow[] = { 2.0f, 16.0f };
	static const float texScaleBox[]  = { 2.0f,  2.0f };
	static int shadow_map = 0;

	shadow_map = (shadow_map) % 6;

	g_pESVDetailStepsMax->SetFloat(2);
	g_pESVCoarseStepsMax->SetFloat(2);
	g_pESVExtrudeFin->SetFloat(0.25f);
    g_pEVVCameraEye->SetFloatVector( (float *) &g_LightPos );
	g_pEVVTexScale->SetFloatVector( (float*) texScaleShadow );
	g_pESRVCubeMapDepth->SetResource( NULL );

	RenderShadowMap(pd3dDevice, 0);
	RenderShadowMap(pd3dDevice, 1);
	RenderShadowMap(pd3dDevice, 2);
	RenderShadowMap(pd3dDevice, 3);
	RenderShadowMap(pd3dDevice, 4);
	RenderShadowMap(pd3dDevice, 5);

    // Clear render target and the depth stencil 
    float ClearColor[4] = { 0.176f, 0.196f, 0.667f, 0.0f };
    pd3dDevice->ClearRenderTargetView( DXUTGetD3D10RenderTargetView(), ClearColor );
    pd3dDevice->ClearDepthStencilView( DXUTGetD3D10DepthStencilView(), D3D10_CLEAR_DEPTH, 1.0, 0 );

	g_pEMVCubeViewMatrix->SetMatrixArray( (float *)g_pmCubeViewMatrices, 0, 6 );
	g_pEMVCubeProjMatrix->SetMatrix( g_mCubeProjectionMatrix );

    D3DXMATRIX  mView = *g_Camera.GetViewMatrix();
    D3DXMATRIX  mProj = *g_Camera.GetProjMatrix();
	D3DXMATRIX mViewProj = mView * mProj;
	g_pEMVViewProj->SetMatrix( (float *) &mViewProj );

    D3DXVECTOR3 mCameraEye = *g_Camera.GetEyePt();
    g_pEVVCameraEye->SetFloatVector( (float* ) &mCameraEye);

	g_pESRVCubeMapDepth->SetResource( g_pSRVCubeMapDepth );
	
	g_pESVDetailStepsMax->SetFloat( g_fDetailStepsMax );
	g_pESVCoarseStepsMax->SetFloat( g_fCoarseStepsMax );
	g_pESVExtrudeFin->SetFloat(g_fExtrudeFin );

	if (g_RenderToHDR) RenderToHDR(pd3dDevice);    

	g_pESRVDiffuse->SetResource     ( g_pSRVDiffuse[MESH] );
    g_pESRVBump->SetResource        ( g_pSRVBump[MESH]    );
    g_pESRVHeight->SetResource      ( g_pSRVHeight[MESH]  );
    
	g_pEVVTexScale->SetFloatVector( (float*) texScaleMesh );
    
	RenderMesh( pd3dDevice );
	
	g_pESRVDiffuse->SetResource     ( g_pSRVDiffuse[CUBE] );
    g_pESRVBump->SetResource        ( g_pSRVBump[CUBE]    );
    g_pESRVHeight->SetResource      ( g_pSRVHeight[CUBE]  );

	g_pEVVTexScale->SetFloatVector( (float*) texScaleBox );
    
	RenderBox ( pd3dDevice );
	
	if (g_RenderToHDR) ShowHDR(pd3dDevice);
 
	RotateLight( (float *) &g_LightPos, g_PauseLight, 0.1f );

	shadow_map++;

	if( g_SettingsDlg.IsActive() )
    {
        g_SettingsDlg.OnRender( fElapsedTime );
        return;
    }

    DXUT_BeginPerfEvent( DXUT_PERFEVENTCOLOR, L "HUD / Stats" );
		RenderText();
		g_HUD.OnRender( fElapsedTime ); 
		g_GUI.OnRender( fElapsedTime );
    DXUT_EndPerfEvent();
}

//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10ResizedSwapChain 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10ReleasingSwapChain( void* pUserContext )
{
	ReleaseHDR();
    g_DialogResourceManager.OnD3D10ReleasingSwapChain();
}

//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10CreateDevice 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10DestroyDevice( void* pUserContext )
{
    g_DialogResourceManager.OnD3D10DestroyDevice();
    g_SettingsDlg.OnD3D10DestroyDevice();

	SAFE_RELEASE( g_pMeshLayout );
    SAFE_RELEASE( g_pFont );
	for (int i = 0; i < MAX_MESH_NUMBER; i++)
	{
		SAFE_RELEASE( g_pT2DDiffuse[i] );
		SAFE_RELEASE( g_pT2DBump[i]    );
		SAFE_RELEASE( g_pT2DHeight[i]  );
		SAFE_RELEASE( g_pSRVDiffuse[i] );
		SAFE_RELEASE( g_pSRVBump[i]    );
		SAFE_RELEASE( g_pSRVHeight[i]  );
	}
	SAFE_RELEASE( g_pMeshEffect );
	
	SAFE_RELEASE( g_pT2DCubeMapDepth );
	SAFE_RELEASE( g_pSRVCubeMapDepth );
	SAFE_RELEASE( g_pDSVCubeMapDepth[0] );
	SAFE_RELEASE( g_pDSVCubeMapDepth[1] );
	SAFE_RELEASE( g_pDSVCubeMapDepth[2] );	
	SAFE_RELEASE( g_pDSVCubeMapDepth[3] );
	SAFE_RELEASE( g_pDSVCubeMapDepth[4] );
	SAFE_RELEASE( g_pDSVCubeMapDepth[5] );

    SAFE_RELEASE( g_pBoxLayout );

    SAFE_DESTROY_MESH( g_pMesh );
	SAFE_DESTROY_MESH( g_pBox  );

    ReleaseHDR();
}

//--------------------------------------------------------------------------------------
// Handle messages to the application
//--------------------------------------------------------------------------------------
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, 
                          bool* pbNoFurtherProcessing, void* pUserContext )
{
    // Pass messages to dialog resource manager calls so GUI state is updated correctly
    *pbNoFurtherProcessing = g_DialogResourceManager.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    // Pass messages to settings dialog if its active
    if( g_SettingsDlg.IsActive() )
    {
        g_SettingsDlg.MsgProc( hWnd, uMsg, wParam, lParam );
        return 0;
    }

    // Give the dialogs a chance to handle the message first
    *pbNoFurtherProcessing = g_HUD.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    *pbNoFurtherProcessing = g_GUI.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    if(g_Camera.HandleMessages( hWnd, uMsg, wParam, lParam ))
    {
        return 1;
    }

    return 0;
}

//--------------------------------------------------------------------------------------
// Handle key presses
//--------------------------------------------------------------------------------------
void CALLBACK OnKeyboard( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext )
{
    if( !bKeyDown )
        return;
    switch (nChar)
    {
        case '1': g_pMeshTechnique = g_pSilhouette;            break;
        case '2': g_pMeshTechnique = g_pReliefMapping;         break;   
        case '3': g_pMeshTechnique = g_pBumpMapping;           break;  

        case 'c': case 'C':
                D3DXVECTOR3 *Eye;
                Eye = (D3DXVECTOR3 *)g_Camera.GetEyePt();
                D3DXVECTOR3 *LookAt;
                LookAt = (D3DXVECTOR3 *)g_Camera.GetLookAtPt();
                Eye = LookAt;
                break;
		case 'b': case 'B': g_pESVBoolean1->SetFloat( g_Boolean1 );
			g_Boolean1 = !( (int)g_Boolean1 );
			break;
		//case 'p': case 'P': g_pESVBoolean2->SetFloat( g_Boolean2 );
		//	g_Boolean2 = !( (int)g_Boolean2 );
		//	break;
		//case 'l': case 'L': g_PauseLight = !g_PauseLight;
		//	break;
		//case 'h': case 'H': g_RenderToHDR = !g_RenderToHDR;
		//	break;
    }
}

//--------------------------------------------------------------------------------------
// Handle mouse button presses
//--------------------------------------------------------------------------------------
void CALLBACK OnMouse( bool bLeftButtonDown, bool bRightButtonDown, bool bMiddleButtonDown, 
                       bool bSideButton1Down, bool bSideButton2Down, int nMouseWheelDelta, 
                       int xPos, int yPos, void* pUserContext )
{
}

//--------------------------------------------------------------------------------------
// Call if device was removed.  Return true to find a new device, false to quit
//--------------------------------------------------------------------------------------
bool CALLBACK OnDeviceRemoved( void* pUserContext )
{
    return true;
}

//--------------------------------------------------------------------------------------
// Initialize everything and go into a render loop
//--------------------------------------------------------------------------------------
int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow )
{
    // Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
    _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

    // DXUT will create and use the best device (either D3D9 or D3D10) 
    // that is available on the system depending on which D3D callbacks are set below
    // Set general DXUT callbacks
    DXUTSetCallbackFrameMove      ( OnFrameMove          );
    DXUTSetCallbackKeyboard       ( OnKeyboard           );
    DXUTSetCallbackMouse          ( OnMouse              );
    DXUTSetCallbackMsgProc        ( MsgProc              );
    DXUTSetCallbackDeviceChanging ( ModifyDeviceSettings );
    DXUTSetCallbackDeviceRemoved  ( OnDeviceRemoved      );

    // Set the D3D10 DXUT callbacks. Remove these sets if the app doesn't need to support D3D10
    DXUTSetCallbackD3D10DeviceAcceptable  ( IsD3D10DeviceAcceptable   );
    DXUTSetCallbackD3D10DeviceCreated     ( OnD3D10CreateDevice       );
    DXUTSetCallbackD3D10SwapChainResized  ( OnD3D10ResizedSwapChain   );
    DXUTSetCallbackD3D10FrameRender       ( OnD3D10FrameRender        );
    DXUTSetCallbackD3D10SwapChainReleasing( OnD3D10ReleasingSwapChain );
    DXUTSetCallbackD3D10DeviceDestroyed   ( OnD3D10DestroyDevice      );

    // IMPORTANT: set SDK media search path to include source directory of this sample, when started from .\Bin
    HRESULT hr;
    V_RETURN(DXUTSetMediaSearchPath(L"..\\Source\\ParallaxMapping"));	
    
	InitDialog();
    
    DXUTInit( true, true, NULL );        // Parse the command line, show msgboxes on error, no extra command line params
    DXUTSetCursorSettings( true, true ); // Show the cursor and clip it when in full screen
    DXUTCreateWindow( L"ParallaxMapping" );
    DXUTCreateDevice( true, 1024, 1024 );  

    DXUTMainLoop(); // Enter into the DXUT render loop

    return DXUTGetExitCode();
}
